\version "2.12.0"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Here begins the scheme functions definitions
%% You can put all this section in a separate file (rhythm.ly for example)

#(define rhythmVectorIndex 0)

#(define (transformEachNote chordElt rhythmVector)
(begin
       (if (or (eq? 'NoteEvent (ly:music-property chordElt 'name))
               (eq? 'RestEvent (ly:music-property chordElt 'name)))
               (set! (ly:music-property chordElt 'duration) (vector-ref rhythmVector rhythmVectorIndex))
       )
       chordElt
))

#(define (getChords musicElt rhythmVector)
(begin
       (if (eq? 'EventChord (ly:music-property musicElt 'name))
               (begin
                       (map
                               (lambda (x) (transformEachNote x rhythmVector))
                               (ly:music-property musicElt 'elements)
                       )
                       (set! rhythmVectorIndex (1+ rhythmVectorIndex))
                       (if (= rhythmVectorIndex (vector-length rhythmVector)) (set! rhythmVectorIndex 0))
               )
       )
       musicElt
))

        %%%%%%%%%%%%%%%% string functions %%%%%%%%%%%%%%%%%%%%%

#(define (string->duration strElt)
(
       let*(
               (ptindex (string-index strElt #\. ))
                                                       ;; position of "." in "4." for exemple. #f if no ".".
               (ptnumber 0)
               (val (string->number (if ptindex (substring strElt 0 ptindex) strElt)))
                                                       ;; val = 1 2 4 8 ... (without the ".")
               (dur (ly:intlog2 val))
                                                       ;; dur = 0 1 2 3 ... (need for ly:make-duration)
       )
                                                       ;; find the number of "." in Duration
   (while ptindex (
               begin
                       (set! ptnumber (1+ ptnumber))
                       (set! ptindex (string-index strElt #\.  (1+ ptindex) ))
                       )
       )
       (ly:make-duration dur ptnumber 1 1)
))



#(define (string->vectorDuration str)
(   let* (
               (i 0)
               (strList (string-split str #\space ))
               (len (length strList))
               (v (make-vector len))
               )
       (map
               (lambda (x)
                       (begin
                               (vector-set! v i (string->duration x))
                               (set! i (1+ i))
                       )
                       x
               )
               strList
       )
       v
))

        %%%%%%%%%%%%%%%%%% the main function %%%%%%%%%%%%%%%%%%%%

makeRhythm  = #(define-music-function (parser location m str) (ly:music? string?)
(       let* (
                       (prevWasSpace #t)
                       (trimmedStr
                               (string-delete
                                       (string-trim-right str)
                                       (lambda (c)
                                               (       let*
                                                       (
                                                               (currentIsSpace (char=? c #\space))
                                                               (res (and prevWasSpace currentIsSpace))
                                                       )
                                                       (if (not res) (set! prevWasSpace currentIsSpace))
                                                       res
                                               )
                                       )
                               )
                       )
               )
       (set! rhythmVectorIndex 0)
       (music-map
               (lambda (x)
                       (getChords x (string->vectorDuration trimmedStr)))
               m
       )
))

%%%%%%% Jetzt kann man dann z.B. Rhythmen einfügen, in dem man vorher einen bestimmten rhythmus definiert. Also, z.B.: so:
%%%%% punktiert = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8. 16"  #})
%%%%%% Und dann kann z.b. eingeben: \punktiert { c d e f e d c d  } 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
zackigschnell = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "16. 32"  #})
#(define SPZ zackigschnell)
zackig = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8. 16"  #})
#(define APS zackig)
% APS =  \zackig 
zackiggemuetlich = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4. 8"  #})
#(define VPA zackiggemuetlich)

zackiglahm = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "2. 4"  #})
#(define HPV zackiglahm)
sarabande = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 4. 8 " #})
#(define VVPA sarabande)
wanderer = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 8 8 " #})
#(define VAA wanderer)
viertelachtel = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 8 " #})
#(define VA viertelachtel)
achtelsechzehntel = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8 16 " #})
#(define AS achtelsechzehntel)
halbeviertel = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "2 4 " #})
#(define HV halbeviertel)
VH = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 2"  #}) %zwischen die Anführungszeichen die Definition eingeben, und vergiss nicht, dem Kind einen Namen zu geben!
viertelvierachtel = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 8 8 8 8 " #})
#(define VAAAA viertelvierachtel)
AV = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8 4 "  #})
ASS = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8 16 16"  #})
APSA = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8. 16 8"  #})
%%% Dabei können auch Artikulationen dabeisein!
%%%%%%%% AUCH Triolen kann man damit einschließen, z.b. \times 2/3 { \langkurz { c d } }
%%%%%%%% oder man definiert einen kombinierten Ryhthmus, bei dem auch xtolen drin sind, wie z.B.: 
 % meinrhythmus = #(define-music-function (parser location musicA musicB musicC)
              % (ly:music? ly:music? ly:music?)
% #{
     % \set tupletSpannerDuration = #(ly:make-moment 1 4)
     % \times 4/5 \makeRhythm $musicA "16 16 16 16 16" 
     % \times 2/3 \makeRhythm $musicB "8 8 8"
     % \makeRhythm $musicC "16 16 16 16 4"
     % \unset tupletSpannerDuration
% #})
 %  
 %%%%%% Wichtig hierbei: $musicA $musicB ,u.s.w.; das kann man dann so in die Partitur einfügen:
	 % \meinrhythmus   { c d e f g } {  a g f } {   e f e d  c }
%%%%%%%%%%%%%%%%%%%%%% Dabei Nicht vergessen: jeder Abschnitt muss in gewschweifte Klammern, sonst klappts nicht!			 %%%



%%%%%%%% sogar dass funktioniert:   \set tupletSpannerDuration = #(ly:make-moment 1 4) \times 2/3  { \viertelachtel { c d  e d e f e d c b a b } } %%%%%%%%%%

%%%%%%%%% Jetzt kommt was, mit dem man sich viel Tipparbeit bei sich gleichförmig wiederholenden Motiven sparen kann!

motiv =
#(define-music-function (parser location name pattern)
   (symbol? ly:music?)
   (letrec ((trans-rec
	    (lambda (pitches)
	     (lambda (music)
	      (let* ((es (ly:music-property music 'elements))
		     (e (ly:music-property music 'element))
		     (p (ly:music-property music 'pitch)))
	
	       (if (pair? es)
		(ly:music-set-property!
		 music 'elements
		 (map (trans-rec pitches) es)))
	
	       (if (ly:music? e)
		(ly:music-set-property!
		 music 'element
		 ((trans-rec pitches) e)))
	
	       (if (ly:pitch? p)
		(let* ((o (ly:pitch-octave p))
		       (n (ly:pitch-notename p))
		       (i (+ (* 7 o) n))
		       (pes (ly:music-property (list-ref pitches i) 'elements))
		       (pnew (ly:music-property (car pes) 'pitch))
		     )
		 (ly:music-set-property! music 'pitch pnew)
	       ))
	       music)))))
    (primitive-eval
     `(define ,name
       (define-music-function (parserb locationb pitchseq)
        (ly:music?)
        (let* ((pitches (ly:music-property pitchseq 'elements))
	      (n       (ly:music-property pitchseq 'name)))
	(if (not (equal? n 'SequentialMusic))
	 (ly:warning "2nd arg of transform must be SequentialMusic, got ~a" n)
	 ((,trans-rec pitches) (ly:music-deep-copy ,pattern)))))))
    (make-music 'SequentialMusic 'void #t)))
%%%%%%%%%% WENN man dann ein Motiv einfuegen will, das sich in immer gleicher Weise wiederholt, dann geht z.b. das mit: 
%%%%%%%%%%% \motiv #'Stefan {c'8 d' c' d' e2} 
%%% Spaeter kann man dann einfuegen: \Stefan {c d e} u.s.w.
%%%% Es ist moeglich, fuer ein Stueck mehrere Motive zu definieren und zu verwenden!
%%%%%% Ausserdem kann man auch "N-tolen" benutzen, sowie diverse Artikulationen, Boegen, Hairpins, Dynamik!
%%%%% Wichtig ist nur, komischerweise, dass man die Toene bei der Definition des Motivs entweder im relative-Mode eingibt oder in der eingestrichenen Oktave
 
    %%%%% Jetzt noch ein paar nützliche Gimmiks, die viel Tipperei sparen:
    t = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  {  $x $x  } {s1*0~ } >>
    #}) 
    %fügt also ganz einfach einen Bogen hinzu und verdoppelt die letzte Note oder Akkord
    viertelsechzehntel = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 16"  #})
% VS =  \viertelsechzehntel 
    #(define VS viertelsechzehntel)
    vs = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \viertelsechzehntel  {     $x $x } } { s4 ~ s16 } >>
    #})  
    %%%%Der Rhythmus kommt ja z.B. in der Apassionata oft vor!
    
    halbepunktviertelpunkt = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "2. 4."  #})
    #(define HPVP halbepunktviertelpunkt)
    hpvp = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \halbepunktviertelpunkt  {     $x $x } } { s2. ~ s4. } >>
    #})  
   
    viertelpunktviertel = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4. 4 "  #})
%    VPV = { \viertelpunktviertel }
    vpv = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \viertelpunktviertel  {     $x $x } } { s4. ~ s4 } >>
    #})  
    VVP = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4 4. "  #})
    vvp = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \VVP  {     $x $x } } { s4 ~ s4. } >>
    #})  
    viertelpunktachtelpunkt = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "4. 8. "  #})
    
    AVP = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music "8 4. "  #})
    
    avp = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \AVP  {     $x $x } } { s8 ~ s4. } >>
    #})  
    vpap = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \viertelpunktachtelpunkt  {     $x $x } } { s4. ~ s8. } >>
    #})  
    %%%%%% einfach die Note verdoppeln und mit tie versehen
    va = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \VA  { $x $x } } { s4 ~ s8 } >>
    #})  
    
    av = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \AV  { $x $x } } { s4 ~ s8 } >>
    #})  
    %Achtel 32tl
    AZ = #(define-music-function (parser location music) (ly:music?) #{  \makeRhythm $music " 8 32 "  #})
     az = #(define-music-function (parser location x) (ly:music?)
    #{
      \context Voice <<  { \AZ  { $x $x } } { s8 ~ s32 } >>
    #})  
%% Ein paar Pausen!
%wichtig ist hier: wenn "r" als erstes steht, soll das eine Pause sein, die vor der Note steht, sonst steht die Pause nach der Note!!
rvp = { r4 r8 }
% Viertel und Achtelpause vor einer Note.
rvp = { r4 r8 }
var = { \rvp }
%oder gerne auch so
%dasselbe nach einer Note
vpr = {  r 8 r4  }
avr = { \vpr }
rav = { \vpr }
%eine Viertel im sechsachteltakt
avpr = { r8  r4 r8  }
rap = { r8 r16 }
RAP = #(define-music-function (parser location x) (ly:music?)
#{
  r8 [ r16  \context Voice << $x {s1*0 ] } >>
#})  
% Viertel und Achtelpause vor einer Note.
rapp = { r8 r16 r32 }
%%%%%%%%dasselbe mit nem Balken noch dazu!
RAPP = #(define-music-function (parser location x) (ly:music?)
#{
  r8 [ r16 r32 \context Voice << $x {s1*0 ] } >>
#})  
appr = { r32 r16 r8 }
ras = { r8 r16 }
asr = { \rap }
%oder gerne auch so
%dasselbe nach einer Note
apr = { r16 r8 }

sar = { \apr }
rsa = { \apr }
fuenfviertelminuseinachtel = { r2 r4 r4 r8 }
fvma = { \fuenfviertelminuseinachtel }
einachtelvonfuenfvierteln = { r8 r2 r2  }
avfv = { \einachtelvonfuenfvierteln }
 



% sieben achtel Pause im Viervierteltakt, nach einer Note
hppr = { r8 r4 r2 }
%dasselbe davor:
rhpp = { r2 r4 r8 }
%fünfzehn sechzehntel Pause
hpppr = { r16 r8 r4 r2 }
%dasselbe genau andersrum:
rhppp = { r2 r4 r8 r16 }

%7/16 Pause im 2/4 Takt
vppr = { r16 r8 r4 }
apr = { r16 r8 } 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   dauer-Xtolen %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%achteltriolen!

AT = #(define-music-function (parser location x) (ly:music?)
#{
  #(override-auto-beam-setting '(end 1 12 3 4) 1 4 'Staff)
  #(override-auto-beam-setting '(end 1 12 3 4) 2 4 'Staff)
  #(override-auto-beam-setting '(end 1 12 3 4) 3 4 'Staff)
  
   #(override-auto-beam-setting '(end 1 12 * *) 1 4)
 #(override-auto-beam-setting '(end 1 12 * *) 2 4) 
 #(override-auto-beam-setting '(end 1 12 * *) 3 4)
#(override-auto-beam-setting '(end 1 12 * *) 4 4)
  \set tupletSpannerDuration = #(ly:make-moment 1 4 )
  \times 2/3 { $x } \unset tupletSpannerDuration
  #(revert-auto-beam-setting '(end 1 12 3 4) 1 4 )
  #(revert-auto-beam-setting '(end 1 12 3 4) 2 4 )
  #(revert-auto-beam-setting '(end 1 12 3 4) 3 4 )
  
   #(revert-auto-beam-setting '(end 1 12 2 2) 1 4 )
  #(revert-auto-beam-setting '(end 1 12 2 2) 2 4 )
  #(revert-auto-beam-setting '(end 1 12 2 2) 3 4 )
   #(revert-auto-beam-setting '(end 1 12 2 2) 4 4 )
#})

%vierteltriolen

VT = #(define-music-function (parser location x) (ly:music?)
#{
  \set tupletSpannerDuration = #(ly:make-moment 1 2)
  \times 2/3 { $x } \unset tupletSpannerDuration 
  
#})  
%sechzehntelquintolen!

SQ = #(define-music-function (parser location x) (ly:music?)
#{   
  %die folgende Zeile sorgt dafür, dass stets nach einem Viertel der Balken zu Ende ist!
  #(override-auto-beam-setting '(end 1 20 * *) 1 4 'Staff)
  #(override-auto-beam-setting '(end 1 20 * *) 2 4 'Staff)
  #(override-auto-beam-setting '(end 1 20 * *) 3 4 'Staff)
  \set tupletSpannerDuration = #(ly:make-moment 1 4)   
  \times 4/5 { $x } \unset tupletSpannerDuration 
#})  
%Achtelquintolen
AQ = #(define-music-function (parser location x) (ly:music?)
#{   
  \set tupletSpannerDuration = #(ly:make-moment 2 4)   
  \times 4/5 { $x } \unset tupletSpannerDuration 
#})  
%achtelsextolen
ASx = #(define-music-function (parser location x) (ly:music?)
#{ %im 44tl-Takt alle 2 Viertel ein Balken
  #(revert-auto-beam-setting '(end 1 12 4 4) 1 4 'Staff)
  #(revert-auto-beam-setting '(end 1 12 4 4) 3 4 'Staff)
  %auch dasselbe im 64tl Takt
  #(override-auto-beam-setting '(end 1 12 6 4) 2 4 'Staff)
  #(override-auto-beam-setting '(end 1 12 6 4) 4 4 'Staff)
  #(override-auto-beam-setting '(end 1 12 6 4) 6 4 'Staff)
  \set tupletSpannerDuration = #(ly:make-moment 1 2)   
  \times 4/6 { $x } \unset tupletSpannerDuration
  #(override-auto-beam-setting '(end 1 12 4 4) 1 4 'Staff)
  #(override-auto-beam-setting '(end 1 12 4 4) 3 4 'Staff)
  %wieder Rückgängig
  #(revert-auto-beam-setting '(end 1 12 6 4) 2 4 'Staff)
  #(revert-auto-beam-setting '(end 1 12 6 4) 4 4 'Staff)
  #(revert-auto-beam-setting '(end 1 12 6 4) 6 4 'Staff)
 
#})
viertelgleichviertel = { \tempo
  \markup {
    \concat { \smaller \general-align #Y #DOWN \note #"4" #1 
      " = "
      \smaller \general-align #Y #DOWN \note #"4" #1 } }
}